#include <iostream>
#include <vector>
#include <unordered_map>
using namespace std;


const bool DEBUG = false;


void runDFS(int x, int y, int tribe);
void printDesert();
void AddColumn(int col_x, int col_y);
void RemoveColumn(int col_x, int col_y);


int n, m, h, w;
const int MAXNMHW = 3 * (10e2);

bool is_populated[MAXNMHW][MAXNMHW];
int tribe[MAXNMHW][MAXNMHW];
bool dfs_used_tribe[MAXNMHW][MAXNMHW];
int curr_tribe = 0;

int num_squares_of_tribe_under_curr_cloud[MAXNMHW];
vector<int> changed_tribes;

unordered_map<int, int> curr_tribes;

int max_num_tribes = 0;


int main() {
	cin >> n >> m >> h >> w;

	for(int y = 0; y < n; ++y)
		for(int x = 0; x < m; ++x) {
			char c;
			cin >> c;
			if(c == 'x')
				is_populated[x][y] = true;
		}

	for(int y = 0; y < n; ++y) {
		for(int x = 0; x < m; ++x) {
			if(is_populated[x][y] && !dfs_used_tribe[x][y]) {
				runDFS(x, y, ++curr_tribe);
			}
		}
	}

	printDesert();

	for(int y = 0; y < n; ++y) {
		for(int x = 0; x < m; ++x) {
			if(x < m - w)
				AddColumn(x, y);
			if(x >= w)
				RemoveColumn(x, y);
			
			int curr_num_tribes = 0;
			for(auto tribe : curr_tribes) {
				if(tribe.second > 0)
					++curr_num_tribes;
			}
			if(curr_num_tribes > max_num_tribes) {
				max_num_tribes = curr_num_tribes;
				if(DEBUG) {
					cout << x << " " << y << endl; 
					for(auto tribe : curr_tribes) {
						if(tribe.second > 0)
							cout << tribe.first << endl;
					}
				}
			}
				
		}
		
		curr_tribes.clear();
	}

	cout << max_num_tribes << endl;

	return 0;
}


void AddColumn(int col_x, int col_y) {
	for(int y = col_y; y < n && y - col_y < h; ++y) {
		if(tribe[col_x][y] != 0)
			++curr_tribes[tribe[col_x][y]];
	}
}

void RemoveColumn(int col_x, int col_y) {
	for(int y = col_y; y < n && y - col_y < h; ++y) {
		if(tribe[col_x][y] != 0)
			++curr_tribes[tribe[col_x][y]];
		--curr_tribes[tribe[col_x][y]];
	}
}


void runDFS(int x, int y, int this_tribe) {
	if(dfs_used_tribe[x][y] || !is_populated[x][y])
		return;

	dfs_used_tribe[x][y] = true;
	tribe[x][y] = this_tribe;

	// left
	if(x > 0)
		runDFS(x - 1, y, this_tribe);

	// right
	if(x < m - 1)
		runDFS(x + 1, y, this_tribe);

	// up
	if(y > 0)
		runDFS(x, y - 1, this_tribe);

	// down
	if(y < n - 1)
		runDFS(x, y + 1, this_tribe);
}


void printDesert() {
	if(!DEBUG)
		return;
	cout << "Desert:" << endl;
	for(int y = 0; y < n; ++y) {
		for(int x = 0; x < m; ++x)
			//cout << (is_populated[x][y] ? "x" : ".") << " ";
			cout << tribe[x][y] << " ";
		cout << endl;
	}
}


/*

10 12 4 5
. x x . . x . . x x x .
. x . . x x . . . x x x
x x x . . x x x x . . .
. . x x . . . . . . x x
x . . x x x x . . . x x
x x . x . . x x . x x .
. x . . x x . x x . x .
. . x . . x x . . x x .
x x x . . . x x x x . .
. x . . x x . . . x . x

*/