#include "shoes.h"

#include <vector>
#include <iostream>
#include <utility>
#include <algorithm>
#include <random>
#include <iomanip>
#include <numeric>

static const int N = 5000 + 3;

static int n, n_tests;
static int other_shoe[2 * N];

static long long number_of_queries;

static bool inside_v[2 * N];
bool pairInSet(const std::vector<int> &v){
    ++number_of_queries;

    for(int x: v){
        if(x < 1 || 2 * n < x || inside_v[x]){
            std::cout << "Your program used pairInSet() on a set that contained duplicates/out of range numbers." << std::endl;
            exit(0);
        }
        inside_v[x] = true;
    }

    int cnt = 0;
    for(int x: v)
        cnt += inside_v[other_shoe[x]];

    for(int x: v)
        inside_v[x] = false;

    return cnt;
}

static long long runOneTest(){
    number_of_queries = 0;

    std::vector<std::pair<int, int>> pairs;
    for(int i = 0; i < n; ++i){
        int a, b;
        //std::cin >> a >> b;
        a=i*2+1;
        b=i*2+2;
        if(a > b){
            std::swap(a, b);
        }
        pairs.push_back({a, b});
        other_shoe[a] = b;
        other_shoe[b] = a;
    }
    std::sort(pairs.begin(), pairs.end());

    std::vector<std::pair<int, int>> pairs_guess = guessPairs(n);
    if(pairs_guess.size() != n){
        std::cout << "Your program returned a vector that contains too many or too few pairs." << std::endl;
        exit(0);
    }

    for(auto [a, b]: pairs_guess){
        if(a < 1 || 2 * n < a || b < 1 || 2 * n < b){
            std::cout << "Your program returned a pair that contains an out of bound number." << std::endl;
            exit(0);
        }
    }

    static bool mentioned[2 * N];
    std::fill(mentioned + 1, mentioned + 1 + 2 * n, false);
    for(auto [a, b]: pairs_guess){
        mentioned[a] = true;
        mentioned[b] = true;
    }

    for(int i = 1; i <= 2 * n; ++i)
        if(!mentioned[i]){
            std::cout << "Your program returned pairs that contain the same shoe." << std::endl;
            exit(0);
        }

    for(int i = 0; i < pairs_guess.size(); ++i){
        if(pairs_guess[i].first > pairs_guess[i].second)
            std::swap(pairs_guess[i].first, pairs_guess[i].second);
    }
    sort(pairs_guess.begin(), pairs_guess.end());

    bool correct = true;
    for(int i = 0; i < pairs_guess.size(); ++i){
        correct &= pairs_guess[i] == pairs[i];
    }

    if(!correct){
        std::cout << "Your program didn't return the correct pairs." << std::endl;
        exit(0);
    }

    return number_of_queries;
}












#include<bits/stdc++.h>
#include "shoes.h"
using namespace std;
const int MAXN=5002;
bool used[2*MAXN];
int mapping[2*MAXN];
mt19937 mt(420);
bool ask(int l,int r,int x)
{
    vector<int>s;
    for(int i=l;i<=r;i++)s.push_back(i);
    s.push_back(x);
    if(s.size()==1)return 0;
    //cout<<l<<" "<<r<<endl;
    return pairInSet(s);
}
vector<pair<int,int> >ret;
vector<int>tmp;
bool ask(vector<int>v)
{
    if(v.size()<2)return 0;
    return pairInSet(v);
}
vector<int> solve(vector<int>v)
{
    if(v.size()==0)return {};
    //cout<<v.size()<<endl;
    shuffle(v.begin(),v.end(),mt);
    if(v.size()==2)
    {
        ret.push_back({v[0],v[1]});
        return {};
    }
    ///important parameter m
    ///try tuning
    //int m=max(1,(int)v.size()/125);
    int m=1;
    while((1<<m)<v.size())m++;
    m=rand()%17+3;
    m=min(m,(int)v.size()-1);
    vector<int>l,r;
    for(int i=0;i<m;i++)
    {
        l.push_back(v[i]);
    }
    for(int i=m;i<v.size();i++)
    {
        r.push_back(v[i]);
    }
    if(l.size()>1&&pairInSet(l))
    {
        l=solve(l);
    }
    if(r.size()>1&&pairInSet(r))
    {
        r=solve(r);
    }
    vector<int>l_nxt;
    for(int i=0;i<l.size();i++)
    {
        int low=0,high=r.size()-1;
        r.push_back(l[i]);
        if(!ask(r))
        {
            r.pop_back();
            l_nxt.push_back(l[i]);
            continue;
        }
        r.pop_back();
        while(low<high)
        {
            int mid=(low+high)/2;
            tmp.clear();
            tmp.push_back(l[i]);
            for(int j=low;j<=mid;j++)
            {
                tmp.push_back(r[j]);
            }
            if(ask(tmp))
            {
                high=mid;
            }
            else low=mid+1;
        }
        ret.push_back({l[i],r[low]});
        tmp.clear();
        for(int j=0;j<low;j++)tmp.push_back(r[j]);
        for(int j=low+1;j<r.size();j++)tmp.push_back(r[j]);
        r=tmp;
    }
    for(auto xd:r)l_nxt.push_back(xd);
    return l_nxt;
}
vector<pair<int,int> >guessPairs(int n)
{
    ret.clear();
    vector<int>v;
    for(int i=1;i<=2*n;i++)
    {
        v.push_back(i);
    }
    solve(v);
    return ret;
    /*vector<int>available;
    for(int i=n+1;i<=2*n;i++)available.push_back(i);
    for(int i=1;i<=n;i++)
    {
        int low=0,high=available.size()-1;
        while(low<high)
        {
            int mid=(low+high)/2;
            if(ask(available[low],available[mid],i))
            {
                high=mid;
            }
            else low=mid+1;
        }
        ret.push_back({i,available[low]});
        vector<int>av1;
        for(int j=0;j<low;j++)
        {
            av1.push_back(available[j]);
        }
        for(int j=low+1;j<available.size();j++)
        {
            av1.push_back(available[j]);
        }
        available=av1;
    }
    return ret;*/
}

























int main(){
    //std::cin >> n >> n_tests;
    n=5000;n_tests=10;
    long double average_queries = 0;
    for(int i = 0; i < n_tests; ++i)
    {
        cout<<i<<endl;
        average_queries += runOneTest();
    }
    average_queries /= n_tests;

    std::cout << std::fixed << std::setprecision(6) << average_queries << std::endl;
}
